/******************************************************************************

  c2c.c
  
  Written by:  Michael Pellegrino
	       Indiana Institute of Technology

  The purpose of this file is to provide the user interface for c2c
  communications.

  Date last edited:  August 4, 1995

  Version:  .12

******************************************************************************/
#include <malloc.h>
#include <nexus.h>
#include <stdio.h>
#include <c2c.h>
#include "c2cdevice.h"

extern Castlist *c2cDataCast;			/* Cast list for all streams */
extern Streamlist *c2cStreams;			/* Stream callback list for all streams */
extern TrackUser **c2cUserList;			/* List of all users to track */
extern Handler hRegister;			/* Register handler */
extern Handler hSession;			/* Session query handler */
extern Handler hUser;				/* Client query handler */
extern Handler hNewSession;			/* Sesssion create handler */
extern Handler hSubscribe;			/* Subscribe handler */
extern Handler hUnregister;			/* Unregister handler */
extern Handler hUnsubscribe;			/* Unsubscribe handler */
extern CaveEnv c2cEnv;				/* Local environment */

/* Prototypes for all callbacks in the c2c library */
void ClientRegisterCallback(void *address, nexus_buffer_t *buffer);
void ClientSessionCallback(void *address, nexus_buffer_t *buffer);
void ClientUserCallback(void *address, nexus_buffer_t *buffer);
void ClientNewSessionCallback(void *address, nexus_buffer_t *buffer);
void ClientSubscribeCallback(void *address, nexus_buffer_t *buffer);
void ClientUnregisterCallback(void *address, nexus_buffer_t *buffer);
void ClientUnsubscribeCallback(void *address, nexus_buffer_t *buffer);
void SubscribeCallback(void *address, nexus_buffer_t *buffer);
void UnsubscribeCallback(void *address, nexus_buffer_t *buffer);
void StreamCallback(void *address, nexus_buffer_t *buffer);

/* Nexus handler definitions */
static nexus_handler_t NexusHandlers[] =
{ /* Handler definitions */
  {
    CLIENT_REGISTER_STRING,CLIENT_REGISTER_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)ClientRegisterCallback
  },   
  {
    CLIENT_UNREGISTER_STRING,CLIENT_UNREGISTER_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)ClientUnregisterCallback
  },   
  {
    CLIENT_SESSION_STRING,CLIENT_SESSION_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)ClientSessionCallback
  },   
  {
    CLIENT_NEWSESSION_STRING,CLIENT_NEWSESSION_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)ClientNewSessionCallback
  },   
  {
    CLIENT_USER_STRING,CLIENT_USER_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)ClientUserCallback
  },   
  {
    CLIENT_SUBSCRIBE_STRING,CLIENT_SUBSCRIBE_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)ClientSubscribeCallback
  },
  {
    CLIENT_UNSUBSCRIBE_STRING,CLIENT_UNSUBSCRIBE_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)ClientUnsubscribeCallback
  },
  {
    SUBSCRIBE_STRING,SUBSCRIBE_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)SubscribeCallback
  },
  {
    UNSUBSCRIBE_STRING,UNSUBSCRIBE_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)UnsubscribeCallback
  },
  {
    STREAM_CALLBACK_STRING,STREAM_CALLBACK_HASH,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)StreamCallback
  },   
  {
    (char *)NULL,0,
    NEXUS_HANDLER_TYPE_NON_THREADED,(nexus_handler_func_t)NULL
  }
}; /* Handler definitions */

/*****************************************************************************  

          Params:  None
         Returns:  N_SUCCESS
  Global Effects:  None

  This function is called when a nexus context is created
    
******************************************************************************/
int NexusBoot(void)
{ /* NexusBoot */
  nexus_register_handlers(NexusHandlers);	/* Register all handlers */
  return(0);					/* Send back as per protocol */
} /* NexusBoot */

/*****************************************************************************  

          Params:  A url
		   A global pointer
         Returns:  N_SUCCESS
  Global Effects:  None

  This function acts as a callback for the nexus_allow_attachments.  It
  broadcasts a global pointer to everyone who attaches.
      
******************************************************************************/
int NexusAllowAttachments(char *url, nexus_global_pointer_t *gp)
{ /* NexusAllowAttachments */
  nexus_global_pointer(gp, NULL);		/* Send back a global pointer */
  return(0);					/* Send along as per protocol */
} /* NexusAllowAttachments */

/*****************************************************************************  

          Params:  Address of global pointer
		   Data buffer
		   Handler name invoked
		   Handler hash
         Returns:  Nothing
  Global Effects:  None

  This callback is called when a handler that is referenced is not registered
  with nexus.
    
******************************************************************************/
void NexusUnknownHandler(void *address,nexus_buffer_t *buffer,
			 char *handler_name,int handler_id)
{ /* NexusUnknownHandler */
  printf("Warning --> unknown handler activated from within c2c library\n\n",
  	 "Handler name: %s\n",
	 "  Hash value: %d\n",handler_name,handler_id);
} /* NexusUnknownHandler */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes register requests on the client side
    
******************************************************************************/
void ClientRegisterCallback(void *address, nexus_buffer_t *buffer)
{ /* ClientRegisterCallback */
  c2cPrintf(120,"awakening sleep in c2cRegister\n");
  nexus_stash_buffer(buffer,&hRegister.buffer);	/* Pack ingredients */
  NexusWake(hRegister);				/* Wake up process */
} /* ClientRegisterCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes register requests on the client side
    
******************************************************************************/
void ClientUnregisterCallback(void *address, nexus_buffer_t *buffer)
{ /* ClientUnregisterCallback */
  c2cPrintf(120,"awakening sleep in c2cUnregister\n");
  NexusWake(hUnregister);			/* Wake up process */
  nexus_stash_buffer(buffer,&hUnregister.buffer); /* Pack ingredients */
} /* ClientUnregisterCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes session inquiries on the client side
    
******************************************************************************/
void ClientSessionCallback(void *address, nexus_buffer_t *buffer)
{ /* ClientSessionCallback */
  c2cPrintf(120,"awakening sleep in c2cGetSessions\n");
  nexus_stash_buffer(buffer,&hSession.buffer);	/* Pack ingredients */
  NexusWake(hSession);				/* Wake up process */
} /* ClientSessionCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes client requests on the client side
    
******************************************************************************/
void ClientUserCallback(void *address, nexus_buffer_t *buffer)
{ /* ClientUserCallback */
  c2cPrintf(120,"awakening sleep in c2cGetClients\n");
  nexus_stash_buffer(buffer,&hUser.buffer);	/* Pack ingredients */
  NexusWake(hUser);				/* Wake up process */
} /* ClientUserCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes client requests on the client side
    
******************************************************************************/
void ClientSubscribeCallback(void *address, nexus_buffer_t *buffer)
{ /* ClientSubscribeCallback */
  c2cPrintf(120,"awakening sleep in c2cSubscribe\n");
  nexus_stash_buffer(buffer,&hSubscribe.buffer);/* Pack ingredients */
  NexusWake(hSubscribe);			/* Wake up process */
} /* ClientSubscribeCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes client requests on the client side
    
******************************************************************************/
void ClientUnsubscribeCallback(void *address, nexus_buffer_t *buffer)
{ /* ClientUnsubscribeCallback */
  c2cPrintf(120,"awakening sleep in c2cUnsubscribe\n");
  nexus_stash_buffer(buffer,&hUnsubscribe.buffer);/* Pack ingredients */
  NexusWake(hUnsubscribe);			/* Wake up process */
} /* ClientUnsubscribeCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes client requests on the client side
    
******************************************************************************/
void ClientNewSessionCallback(void *address, nexus_buffer_t *buffer)
{ /* ClientNewSessionCallback */
  c2cPrintf(120,"awakening sleep in c2cSessionCreate\n");
  nexus_stash_buffer(buffer,&hNewSession.buffer); /* Pack ingredients */
  NexusWake(hNewSession);			/* Wake up process */
} /* ClientNewSessionCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes client requests on the client side
    
******************************************************************************/
void SubscribeCallback(void *address, nexus_buffer_t *buffer)
{ /* SubscribeCallback */
  nexus_buffer_t buf;			/* Buffer for send data */
  nexus_global_pointer_t gp;		/* Who to call with data */
  StreamType stream;			/* Type of stream to send */
  CAVEId datasource;			/* Id of the client as seen by the remote process */ 
  Castlist *list;			/* Work pointer */
  NexusAddr *addr;			/* Work pointer */
  int status = E_SUCCESS;		/* Status indicator */
  StreamP streamcallback;        /* Callback pointer */

  c2cPrintf(120,"interrupted by broker for subscription\n");
  nexus_get_global_pointer(buffer,&gp,1);	/* Get who to cast to */
  nexus_get_int(buffer,&datasource,1);		/* Get CAVEId of remote process */
  nexus_get_int(buffer,&stream,1);		/* Get stream type */
  for(list = c2cDataCast;		/* Find our stream table */
      (list != NULL) && (list->type != stream);
      list = list->next);
  if(list != NULL)			/* Stream is in list so add gp to it */
  { /* If */
    c2cPrintf(120,"adding subscriber to the list\n");
    addr = (NexusAddr *)malloc(sizeof(NexusAddr)); /* Get some memory */
    addr->next = list->conntab.addr;	/* Insert into cast list */
    list->conntab.addr = addr;
    list->conntab.count++;		/* Count the new addition to list */
    if(list->autofrequency)		/* Update frequency? */
    { /* If */
      list->frequency = list->conntab.count;	/* Set skips to # cast to */
    } /* If */
    list->conntab.addr->caveid = datasource;	/* Set id of remote cave */
    list->type = stream;		/* Set stream type */
    memcpy(&addr->gp,&gp,sizeof(nexus_global_pointer_t));/* Copy gp into table */
  } /* If */
  else					/* No stream of requested type in list */
  { /* Else */
    status = E_INVALID_STREAM;	        /* Set error flag */
  } /* Else */
  c2cPrintf(120,"sending subscriber result %d\n",status);
  nexus_init_remote_service_request(&buf,&gp, /* Send back status of operation */
				    CLIENT_SUBSCRIBE_STRING,
				    CLIENT_SUBSCRIBE_HASH);
  nexus_set_buffer_size(&buf,nexus_sizeof_int(&buf,1),1);
  nexus_put_int(&buf,&status,1);
  nexus_send_remote_service_request(&buf); /* Send along to remote process */
  c2cPrintf(120,"finished sending message to subscriber\n");
  if((list->callback != NULL) && (status == E_SUCCESS)) /* Subscribe indicator? */
  { /* If */
    c2cPrintf(120,"executing stream subscribe callback\n");
    streamcallback = (StreamP)list->callback; /* Cast approproately */
    streamcallback(stream,datasource);    /* Execute indicator callback */
  } /* If */
  c2cPrintf(120,"finished in subscription interrupt\n");
} /* SubscribeCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Entry is added client table

  This callback processes client requests on the client side
    
******************************************************************************/
void UnsubscribeCallback(void *address, nexus_buffer_t *buffer)
{ /* UnsubscribeCallback */
  nexus_buffer_t buf;			/* Data buffer */
  nexus_global_pointer_t gp;		/* Who to call with data */
  StreamType stream;			/* Type of stream to send */
  Castlist *list;			/* Work pointer */
  NexusAddr *prevaddr,*curaddr;		/* Work pointers */
  int status = E_SUCCESS;		/* Status of operation */

  c2cPrintf(120,"inside unsubscribe interrupt\n");
  nexus_get_global_pointer(buffer,&gp,1);	/* Get who to cast to */
  nexus_get_int(buffer,&stream,1);		/* Get stream type */
  for(list = c2cDataCast;		/* Find our stream table */
      (list != NULL) && (list->type != stream);
      list = list->next);
  if(list != NULL)			/* Stream is in the list so remove gp */
  { /* If */
    prevaddr = list->conntab.addr;	/* Point to beginning of cast list */
    if(nexus_same_global_pointer(&prevaddr->gp,&gp)) /* First node match? */
    { /* If */
      c2cPrintf(120,"removing subscriber from list\n");
      list->conntab.addr = prevaddr->next;	/* Update links in list */
      free(prevaddr);			/* Free node from memory */
    } /* If */
    else
    { /* Else */
      for(curaddr = prevaddr->next;	/* Prepare for list traversal */
          (curaddr != NULL) && (!nexus_same_global_pointer(&curaddr->gp,&gp));
	  curaddr = curaddr->next, prevaddr = prevaddr->next);
      if(curaddr != NULL)		/* Found it-->erase it */
      { /* If */
        c2cPrintf(120,"removing subscriber from list\n");
	prevaddr->next = curaddr->next;	/* Remove links from table */
	free(curaddr);			/* Free memory */
      } /* If */
      else				/* Not found in list */
      { /* Else */
        status = E_INVALID_STREAM;	/* Set error message */
      } /* Else */
    } /* Else */
  } /* If */
  else					/* No stream of requested type in list */
  { /* Else */
    status = E_INVALID_STREAM;		/* Set error message */
  } /* Else */
  c2cPrintf(120,"sending unsubscriber result of %d\n",status);
  nexus_init_remote_service_request(&buf,&gp,	/* Send status back */
				    CLIENT_UNSUBSCRIBE_STRING,
				    CLIENT_UNSUBSCRIBE_HASH);
  nexus_set_buffer_size(&buf,nexus_sizeof_int(&buf,1),1);
  nexus_put_int(&buf,&status,1);
  nexus_send_remote_service_request(&buf); /* Send along to remote process */
  c2cPrintf(120,"finished sending unsubscriber the result\n");
} /* UnsubscribeCallback */

/*****************************************************************************  

          Params:  Address portion of global pointer
		   Data buffer
         Returns:  None
  Global Effects:  Particular callback is executed for client data

  This callback parses incoming stream data and executes all callbacks
  associated with each stream of data
    
******************************************************************************/
void StreamCallback(void *address, nexus_buffer_t *buffer)
{ /* StreamCallback */
  StreamType streamtype;		/* What stream are we processing */
  nexus_global_pointer_t gp;		/* Buffer for incoming global pointer */
  Streamlist *list;			/* Work pointer */
  CAVEId id;				/* Sending id of CAVE */
  float locx,locy,locz,		        /* Locations of object */
        anga,ange,angr;		        /* Orientation angles */
  TrackerP tracker;			/* Tracker callback */
  ButtonP Button;			/* Button callback */
  int but1,but2,but3,but4;		/* Wand buttons */
  JoystickP Joystick;			/* Joystick callback */
  float joyx,joyy;			/* Joystick values */
  CAVEUserP UserCallback;		/* CAVEUser callback */
  CAVEUser User;			/* Data buffer */
  UserDataP Data;			/* UserData callback */
  nexus_stashed_buffer_t buf;		/* Data buffer */

  nexus_get_global_pointer(buffer,&gp,1);	/* Get remote global pointer */
  nexus_get_int(buffer,&id,1);		/* Read CAVEId */
  nexus_get_int(buffer,&streamtype,1);	/* Read type of stream interrupting */
  c2cPrintf(120,"processing incoming stream data from %d of type %d\n",id,streamtype);
  for(list = c2cStreams;		/* Find our stream table */
      (list != NULL) && (list->type != streamtype);
      list = list->nexttype);
  if(list != NULL)			/* Not registered in the list */
  { /* If */
    for( /* No starting condition */ ;	/* Traverse for all callbacks */
        list != NULL;			/* End when list ends */
        list = list->nextfunc)		/* Advance every node */
    { /* For loop */
      if(list->source == id)		/* Process data if intended for this node in list */
      { /* If */
        switch(streamtype)		/* Where is it to go? */
        { /* Switch */
          case S_HEAD_TRACKER:		/* Head tracker info */
          case S_WAND_TRACKER:		/* Wand tracker info */
          case S_WORLD_POSITION:		/* World info */
            c2cPrintf(120,"reading a tracker stream\n");
	    nexus_get_float(buffer,&locx,1);	/* Read location value */
	    nexus_get_float(buffer,&locy,1);	/* Read location value */
	    nexus_get_float(buffer,&locz,1);	/* Read location value */
	    nexus_get_float(buffer,&anga,1);	/* Read angle value */
	    nexus_get_float(buffer,&ange,1);	/* Read angle value */
	    nexus_get_float(buffer,&angr,1);	/* Read angle value */
	    tracker = (TrackerP)list->callback;	/* Cast the callback */
	    tracker(locx,locy,locz,anga,ange,angr,id);
            break;
          case S_WAND_BUTTON:		/* Button information */
            c2cPrintf(120,"reading wand button stream\n");
	    nexus_get_int(buffer,&but1,1); /* Read the button value */
	    nexus_get_int(buffer,&but2,1); /* Read the button value */
	    nexus_get_int(buffer,&but3,1); /* Read the button value */
	    nexus_get_int(buffer,&but4,1); /* Read the button value */
	    Button = (ButtonP)list->callback; /* Cast the callback */
	    Button(but1,but2,but3,but4,id); /* Execute the callback */
            break;
          case S_WAND_JOYSTICK:		/* Joystick information */
            c2cPrintf(120,"reading wand joystick stream\n");
	    nexus_get_float(buffer,&joyx,1); /* Read the joystick value */
	    nexus_get_float(buffer,&joyy,1); /* Read the joystick value */
	    Joystick = (JoystickP)list->callback; /* Cast the callback */
	    Joystick(joyx,joyy,id); /* Execute the callback */
            break;
	  case S_CAVE_USER:		/* CAVE user */
            c2cPrintf(120,"reading CAVE user stream\n");
	    nexus_get_char(buffer,(char *)&User,sizeof(CAVEUser)); /* Read info */
	    UserCallback = (CAVEUserP)list->callback; /* Cast the callback */
	    UserCallback(&User,id);	/* Execute the callback */
	    break;
          default:                      /* Any user data stream */
            c2cPrintf(120,"reading a user defined stream %d\n",streamtype);
	    nexus_stash_buffer(buffer,&buf); /* Move data to static location */
	    Data = (UserDataP)list->callback; /* Cast the callback */
	    Data((void *)&buf,id);	/* Execute the callback */
            break;
        } /* Switch */
      } /* If */
    } /* For loop */
  } /* If */
  else					/* Error-->stream not in the list */
  { /* Else */
    printf("Warning -->  Received data for an unregistered stream\n",
           "Sender CAVEId:  %d\n",
	   "   StreamType:  %d\n",id,streamtype);
  } /* Else */
  c2cPrintf(120,"done processing data in stream callback\n");
} /* StreamCallback */

/*****************************************************************************  

          Params:  None
         Returns:  None
  Global Effects:  Shared memory arena specific to CAVE-to-CAVE library is
                   created

  This function serves as the default callback when c2cTrackUserInit is used.
  What it does basically is stuff the CAVEUser parameters into the specific
  node in the shared memory linked list.
    
******************************************************************************/
void c2cTrackUserCallback(CAVEUser *user,int id)
{ /* c2cTrackUserCallback */
  TrackUser *curuser;			/* Work pointer */

  c2cPrintf(120,"received data from %d to put in list\n",id);
  for(curuser=*c2cUserList;		/* Find the user who the data belongs to */
      (curuser != NULL) && (curuser->env.id != id);
      curuser = curuser->next);
  if(curuser != NULL)			/* Make sure the user is in the list */
  { /* If */
    curuser->user.headx = user->headx;	/* Copy all data accordingly */
    curuser->user.heady = user->heady;
    curuser->user.headz = user->headz;
    curuser->user.heada = user->heada;
    curuser->user.heade = user->heade;
    curuser->user.headr = user->headr;
    curuser->user.wandx = user->wandx;
    curuser->user.wandy = user->wandy;
    curuser->user.wandz = user->wandz;
    curuser->user.wanda = user->wanda;
    curuser->user.wande = user->wande;
    curuser->user.wandr = user->wandr;
    curuser->user.worldx = user->worldx;
    curuser->user.worldy = user->worldy;
    curuser->user.worldz = user->worldz;
    curuser->user.worlda = user->worlda;
    curuser->user.worlde = user->worlde;
    curuser->user.worldr = user->worldr;
    curuser->user.joyx = user->joyx;
    curuser->user.joyy = user->joyy;
    curuser->user.but1 = user->but1;
    curuser->user.but2 = user->but2;
    curuser->user.but3 = user->but3;
    curuser->user.but3 = user->but4;
  } /* If */
  else					/* User is not in the list */
  { /* Else */
    printf("Warning -->  c2cTrackUserCallback received data from CAVEId %d\n",
           "             that is not in its tracking list.",id);
  } /* Else */
  c2cPrintf(120,"finished processing user tracking data\n");
} /* c2cTrackUserCallback */
