/******************************************************************************

  c2config.c
  
  Written by:  Michael Pellegrino
	       Indiana Institute of Technology

  The purpose of this file is to provide handle configuration file operations

  Date last edited:  August 4, 1995

  Version:  .12

******************************************************************************/
#include <ctype.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <c2c.h>
#include "c2cdevice.h"

/*****************************************************************************  

          Params:  Nothing
         Returns:  Nothing
  Global Effects:  None

  All c2c functionality is terminated
  
******************************************************************************/
int c2cReadConfigFile(char *filename,c2cConfig *config)
{ /* c2cReadConfigFile */
  FILE *fp;				/* Config file pointer */
  char *name;				/* Filename buffer */
  char buffer[CONFIG_FILE_LINE_SIZE];	/* Line buffer */
  char *option;				/* Config option string */

  c2cPrintf(110,"attempting to process c2c configuration file name %s\n",filename);
  if(filename == NULL)			/* Any filename given? */
  { /* If */
    name = (char *)malloc(strlen(C2C_CONFIG_FILE) + 1); /* Get some memory */
    strcpy(name,C2C_CONFIG_FILE);	/* Set a default file to check */
    c2cPrintf(110,"trying default configuration filename %s\n",name);
  } /* If */
  else					/* Just point to the filename */
  { /* Else */
    name = filename;			/* Point to the given filename */
  } /* Else */
  c2cPrintf(110,"attempting to open %s as a configuration file\n",name);
  fp = fopen(name,"r");			/* Open the file */
  if(fp != NULL)			/* File opened */
  { /* If */
    while(fgets(buffer,CONFIG_FILE_LINE_SIZE,fp)) /* As long as there is data to read */
    { /* While */
      option = strtok(buffer," \t\n"); /* Get config option */
      if((buffer[0] != '#') && (option != NULL)) /* Ignore comments and blank lines */
      { /* If */
	c2cLowerCase(option);		/* Convert to all lowercase */
        c2cPrintf(120,"processing config file option %s\n",option);
	if(strcmp(option,"broker") == 0) /* Broker url */
	{ /* If */
	  strcpy(config->broker,strtok(NULL,"\"")); 
	} /* If */
	else if(strcmp(option,"clientapptype") == 0) /* Parse application type */
	{ /* If */
	  option = strtok(NULL," \t\n"); /* Get app type */
	  c2cLowerCase(option);
	  if(strcmp("cave",option) == 0)
	  { /* If */
	    config->env.type = ST_CAVE;
	  } /* If */
	  else if (strcmp("simulator",option) == 0)
	  { /* If */
	    config->env.type = ST_CAVE_SIMULATOR;
	  } /* If */
	  else if (strcmp("immersadesk",option) == 0)
	  { /* If */
	    config->env.type = ST_IMMERSADESK;
	  } /* If */
	  else if (strcmp("simulation",option) == 0)
	  { /* If */
	    config->env.type = ST_SIMULATION;
	  } /* If */
	  else				/* Don't know type */
	  { /* Else */
	    printf("Warning --> Unknown clienttype %s in %s\n",option,name);
	  } /* Else */
	} /* If */
	else if(strcmp(option,"clientname") == 0) /* Client name */
	{ /* If */
	  strcpy(config->env.name,strtok(NULL,"\t\n\""));
	  c2cPrintf(120,"client name read: %s\n",config->env.name);
	} /* If */
	else if(strcmp(option,"debuglevel") == 0) /* debug level */
	{ /* If */
	  option = strtok(NULL," \t\n"); /* Get app type */
	  c2cSetDebugLevel(atoi(option));
	} /* If */
	else if(strcmp(option,"machinemodel") == 0) /* Machine model */
	{ /* If */
	} /* If */
	else if(strcmp(option,"machinemake") == 0) /* Machine make */
	{ /* If */
	} /* If */
	else if(strcmp(option,"sessionargs") == 0) /* Session arguments */
	{ /* If */
	  strcpy(config->session.args,strtok(NULL,"\t\n\"")); 
	} /* If */
	else if(strcmp(option,"sessionexe") == 0) /* Session executable */
	{ /* If */
	  strcpy(config->session.execname,strtok(NULL,"\t\n\"")); 
	} /* If */
	else if(strcmp(option,"sessionname") == 0) /* Session name */
	{ /* If */
	  strcpy(config->session.name,strtok(NULL,"\t\n\"")); 
	} /* If */
	else if(strcmp(option,"sessionpath") == 0) /* Session pathname */
	{ /* If */
	  strcpy(config->session.pathname,strtok(NULL,"\t\n\"")); 
	} /* If */
	else				/* Unknown option */
	{ /* Else */
	  printf("Warning --> unknown option %s in %s configuration file\n",option,name);
	} /* Else */
      } /* If */
      else                              /* Blank line or comment */
      { /* Else */
        c2cPrintf(120,"skipping blank or comment line\n");
      } /* Else */
    } /* While */
    fclose(fp);				/* Close the configuration file */
    c2cPrintf(110,"finished reading configuration file %s\n",name);
  } /* If */
  else					/* Error on file open */
  { /* Else */
    c2cPrintf(110,"error in opening configuration file %s\n",name);
    return(E_OPEN_CONFIG_FILE);         /* Send back error */
  } /* Else */
  return(E_SUCCESS);                    /* Send back success */
} /* c2cReadConfigFile */
