/******************************************************************************

  c2cgraphics.c
  
  Written by:  Michael Pellegrino
	       Indiana Institute of Technology

  The purpose of this file is to provide the graphics routines for the library

  Date last edited:  August 4, 1995

  Version:  .12

******************************************************************************/
#ifdef C2C_GL_GRAPHICS
#include <gl/gl.h>
#include <gl/sphere.h>
#endif
#ifdef C2C_OPENGL_GRAPHICS
#include <GL/gl.h>
#include <GL/glu.h>
#endif
#include <c2c.h>
#include "c2cdevice.h"

/******************************************************************************

         Params:  Location/orientation of user and wand in CAVE
   Return Value:  Nothing
 Global Effects:  None

 This function draws a stick figure and a wand.  The wand code was taken from
 the CAVE library and was written by Dave Pape and Carolina Cruiz-Niera at the 
 Electronic Visualization Lab at the University of Illinois at Chicago

******************************************************************************/
void c2cCAVEMan(float Hx,float Hy,float Hz,float Ha,float He,float Hr,
                float wx,float wy,float wz,float wa,float we,float wr)
{ 
  float params[4];			/* Point buffers */
  float points[3];
  static float
    wand_v1[3] = {-WAND_END_OFFSET,    WAND_END_OFFSET,   -WAND_LENGTH},
    wand_v2[3] = {-WAND_START_OFFSET,  WAND_START_OFFSET,  WAND_LENGTH},
    wand_v3[3] = { WAND_END_OFFSET,    WAND_END_OFFSET,   -WAND_LENGTH},
    wand_v4[3] = { WAND_START_OFFSET,  WAND_START_OFFSET,  WAND_LENGTH},
    wand_v5[3] = { WAND_END_OFFSET,   -WAND_END_OFFSET,   -WAND_LENGTH},
    wand_v6[3] = { WAND_START_OFFSET, -WAND_START_OFFSET,  WAND_LENGTH},
    wand_v7[3] = {-WAND_END_OFFSET,   -WAND_END_OFFSET,   -WAND_LENGTH},
    wand_v8[3] = {-WAND_START_OFFSET, -WAND_START_OFFSET,  WAND_LENGTH};

#ifdef C2C_GL_GRAPHICS
  RGBcolor(255,0,0);		/* Yellow user */
  linewidth(8);
  pushmatrix();
    translate(Hx,Hy,Hz);	/* Go to head location and draw */
    rot(Ha,'y');	/* Rotate body as per head orientation */

    pushmatrix();	/* Draw head */
      rot(He,'x');  /* Rotate X,Z only for head so body doesn't follow */
      rot(Hr,'z');
      params[0] = params[1] = params[2] = 0;
      params[3] = .5;
      sphdraw(params);
      cpack(0xff000000);	/* Black for eyes */
      translate(.125,.125,-.5);
        params[3] = .0875;
        sphdraw(params);
        translate(-.25,0,0);
        sphdraw(params);
    popmatrix();

    cpack(0xf00fffff);	/* Make body yellow */
    bgnline();	/* Body */
      points[0] = points[1] = points[2] = 0;
      v3f(points);
      points[1] = Hy * -.45;
      v3f(points);
      points[0] -= 1;
      points[1] = (-1)*Hy;
      v3f(points);
    endline();
    bgnline();	/* Legs */
      points[0] += 2;
      v3f(points);
      points[0] -= 1;
      points[1] = Hy * -.45;
      v3f(points);
    endline();
    bgnline();
      points[0] = .5;	/* Shoulders */
      points[1] = Hy * -.15;
      points[2] = 0;
      v3f(points);
      points[0] -= 1;
      v3f(points);
      points[0] -= .5; /* Other arm */
      points[1] = Hy * -.27;
      v3f(points);
      points[0] += .5;
      points[1] = Hy * -.40;
      v3f(points);
    endline();
    pushmatrix();	
      translate(.5,Hy * -.15,0);
      bgnline();
        points[0] = points[1] = points[2] = 0;
        v3f(points);
        points[0] = wx - .5 -Hx;
        points[1] = wy - (Hy*-.15) - Hy;
        points[2] = wz - Hz;
        v3f(points);
        endline(); 
    popmatrix();
  popmatrix();

  /* The following is Dave Pape's code */
  pushmatrix();			/* Draw the wand */
    translate(wx,wy,wz);
    rot(wa,'y');
    rot(we,'x');
    rot(wr,'z');
    bgntmesh();
      cpack(0xff00ff00);
      v3f(wand_v1);
      v3f(wand_v2);
      v3f(wand_v3);
      v3f(wand_v4);
      cpack(0xff00f000);
      v3f(wand_v5);
      v3f(wand_v6);
      cpack(0xff00ff00);
      v3f(wand_v7);
      v3f(wand_v8);
      cpack(0xff00f000);
      v3f(wand_v1);
      v3f(wand_v2);
    endtmesh();
  popmatrix();
#endif

#ifdef C2C_OPENGL_GRAPHICS
	GLint slices = 40;
	GLint stacks = 40;
	GLUquadricObj *sphere = gluNewQuadric();

	gluQuadricOrientation(sphere,GLU_OUTSIDE);
	gluQuadricNormals(sphere,GLU_FLAT);
	gluQuadricDrawStyle(sphere,GLU_FILL);

	glPushAttrib( GL_ALL_ATTRIB_BITS );
	glPushMatrix();
		glTranslatef(Hx, Hy, Hz);/* Move to head location and draw */
		glRotatef(Ha,0.0,1.0,0.0);	       /* Rotate body as per head orientation */
		glPushMatrix();							/* Draw head */
			glRotatef(He,1.0,0.0,0.0);	  
			glRotatef(Hr,0.0,0.0,1.0);
			glColor3f( 1, 0, 0 );				    	/* Red head	*/
			gluSphere(sphere,0.5,slices,stacks);
			glTranslatef(.125, .125, -0.5);
			glColor3f( 0, 0, 0 );				  	/* Black eyes	*/
			gluSphere(sphere,0.0875,slices,stacks);
			glTranslatef(-0.25, 0, 0);
			gluSphere(sphere,0.0875,slices,stacks);
		glPopMatrix();			
		glColor3f( 1, 1, 0 );
		glLineWidth( 8 );
		glBegin( GL_LINE_STRIP );				   /* Body and right leg */
			glVertex3f( 0, 0, 0 );
			glVertex3f(  0,  Hy * -0.45, 0 );
			glVertex3f( -1, -Hy, 0 );
		glEnd();
		glBegin( GL_LINES );							/* Legs */
			glVertex3f( 1, -Hy, 0 );
                        glVertex3f( 0,  Hy * -0.45, 0 );
		glEnd();
		glBegin( GL_LINE_STRIP );	     	/* Shoulders and left arm */
			glVertex3f(  0.5, Hy * -0.15, 0 );
			glVertex3f( -0.5,  Hy * -0.15, 0 );
			glVertex3f( -1, Hy * -0.27, 0 );
			glVertex3f( -0.5,  Hy * -0.40, 0 );
		glEnd();
		glPushMatrix();
		glTranslatef( 0.5, Hy * -0.15, 0 );
		glBegin( GL_LINES );			  	 /* Shoulders and left arm */
			glVertex3f(  0, 0, 0 );
			glVertex3f( wx - Hx - 0.5, 
					  wy - Hy - (Hy * -0.15), 
					  wz - Hz );	
		glEnd();
		glPopMatrix();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(wx, wy, wz);
		glRotatef(wa,0.0,1.0,0.0);
		glRotatef(we,1.0,0.0,0.0);
		glRotatef(wr,0.0,0.0,1.0);
		glBegin ( GL_TRIANGLE_STRIP );
			glColor3f( 1, 0, 0 );
			glVertex3fv( wand_v1 );
			glVertex3fv( wand_v2 );
			glVertex3fv( wand_v3 );
			glVertex3fv( wand_v4 );
			glColor3f( 0.5, 0, 0 );
			glVertex3fv( wand_v5 );
			glVertex3fv( wand_v6 );
			glColor3f( 1, 0, 0 );
			glVertex3fv( wand_v7 );
			glVertex3fv( wand_v8 );
			glColor3f( 0.5, 0, 0 );
			glVertex3fv( wand_v1 );
			glVertex3fv( wand_v2 );
		glEnd();
		glColor3f( 1.0, 1.0, 0.0 );
		glLineWidth( 1 );
		glBegin( GL_LINES );				       		/* Joystick on wand */
			glVertex3f( 0,  0.1, 0 ); 
			glVertex3f( 0,  0.2, 0 ); 
		glEnd();
	glPopMatrix();
	glPopAttrib();

#endif

} /* c2cCAVEMan */

/*****************************************************************************  

          Params:  CAVEUser
         Returns:  None
  Global Effects:  None

  This function draws a user based on the parameters
    
******************************************************************************/
void c2cDrawStickMan(CAVEUser *user,CAVEId id)
{ /* c2cDrawStickMan */
  c2cPrintf(120,"starting drawing stick man of a user\n");
  c2cCAVEMan(user->headx,user->heady,user->headz,
             user->heada,user->heade,user->headr,
	     user->wandx,user->wandy,user->wandz,
	     user->wanda,user->wande,user->wandr);
  c2cPrintf(120,"finishing drawing stick man of a user\n");
} /* c2cDrawStickMan */
