/******************************************************************************

  c2cutil.c
  
  Written by:  Michael Pellegrino
	       Indiana Institute of Technology

  The purpose of this file is to provide various utilities functions to
  the c2c library

  Date last edited:  August 4, 1995

  Version:  .12

******************************************************************************/
#include <ctype.h>
#include <c2c.h>
#include <p2p.h>
#include <stdarg.h>
#include "c2cdevice.h"

extern int c2cDebugLevel;               /* Debugging information value */

/*****************************************************************************  

          Params:  String
         Returns:  Lowercase string
  Global Effects:  None

  This function converts a string to all lower-case characters
  
******************************************************************************/
void c2cLowerCase(char *str)
{ /* c2cLowerCase */
  while(*str)
  { /* While */
    *str = tolower(*str);		/* Convert the character */
    str++;				/* Advance to next character */
  } /* While */
} /* c2cLowerCase */

/*****************************************************************************  

          Params:  None
         Returns:  Nothing
  Global Effects:  Shared memory arena is set up

  This function sets up a shared memory arena
  
******************************************************************************/
void c2cSharedInit(void)
{ /* c2cSharedInit */
  p2p_init(1,SHARED_MEM_SIZE);          /* Set up shared memory arena */
} /* c2cSharedInit */

/*****************************************************************************  

          Params:  Size of block to allocate
         Returns:  Pointer to the block
  Global Effects:  None

  This function allocates a block of shared memory
  
******************************************************************************/
void *c2cSharedAlloc(int size)
{ /* c2cSharedAlloc */
  void *ptr;                            /* Data pointer */

  ptr = (void *)p2p_shmalloc(size);             /* Get memory */
  return(ptr);           /* Return pointer */
} /* c2cSharedAlloc */

/*****************************************************************************  

          Params:  Pointer to block to free
         Returns:  Nothing
  Global Effects:  None

  This function frees a block of shared memory
  
******************************************************************************/
void c2cSharedFree(void *ptr)
{ /* c2cSharedFree */
  p2p_shfree((char *)ptr);                      /* Free shared memory */
} /* c2cSharedFree */

/*****************************************************************************  

          Params:  None
         Returns:  Nothing
  Global Effects:  None

  This function exits all shared memory functions
  
******************************************************************************/
void c2cSharedExit(void)
{ /* c2cSharedExit */
  p2p_cleanup();                        /* Clean up all shared memory stuff */
} /* c2cSharedExit */

/*****************************************************************************  

          Params:  Variable
         Returns:  Nothing
  Global Effects:  None

  This function prints out a string based on a the first a level.  If the
  level supplied is greater than the global variable c2cDebugLevel, then the
  function returns, otherwise, it is printed.
  
******************************************************************************/
void c2cPrintf(int level,char *format, ...)
{ /* c2cPrintf */
  va_list args;                         /* Variable argument list */

  if(level <= c2cDebugLevel)            /* Can we print the text? */
  { /* If */
    printf("c2cdbg %d: ",level);        /* Print out level indicator */
    va_start(args,format);              /* Start parsing... */
    vprintf(format,args);               /* Print everything out */
    va_end(args);                       /* End argument parsing */
  } /* If */
} /* c2cPrintf */

/*****************************************************************************  

          Params:  Debugging level value
         Returns:  Nothing
  Global Effects:  Debugging value is set

  This function sets the c2c debugging level value
  
******************************************************************************/
void c2cSetDebugLevel(int level)
{ /* c2cSetDebugLevel */
  c2cDebugLevel = level;                        /* Set the debugging level */
} /* c2cSetDebugLevel */

/*****************************************************************************  

          Params:  None
         Returns:  Debugging value
  Global Effects:  None

  This function gets the c2c debugging level value
  
******************************************************************************/
int c2cGetDebugLevel(void)
{ /* c2cGetDebugLevel */
  return(c2cDebugLevel);                        /* return the debugging level */
} /* c2cGetDebugLevel */
