/*
 * initwrap.c
 *
 * This wraps the nexus initialization routine, _nx_init().
 * Under AIX, this must be statically linked into an application
 * so that libnexus can be dynamically linked with no
 * unresolved symbols.
 */

static char *rcsid = "$Header: /Net/folding.compbio.caltech.edu/bcomp/src/master/nexus/initwrap.c,v 1.9 1995/06/06 23:48:12 tuecke Exp $";

#include "internal.h"

/*
 * Define a default module_list
 */
#ifdef NEXUS_ARCH_PARAGON
#define DEFINED_MODULE_LIST
#ifndef NEXUS_LITE
extern void *_nx_st_inx_info();
#endif /* NEXUS_LITE */
extern void *_nx_pr_inx_info();
extern void *_nx_database_file_info();
static nexus_module_list_t default_module_list[] =
{
#ifndef NEXUS_LITE
    {"startups", "inx", _nx_st_inx_info},
#endif /* NEXUS_LITE */
    {"protocols", "inx", _nx_pr_inx_info},
    {"database", "file", _nx_database_file_info},
    {NULL, NULL, NULL},
};
#endif
#ifdef NEXUS_ARCH_MPL
#define DEFINED_MODULE_LIST
#ifndef NEXUS_LITE
#ifdef NEXUS_ARCH_STARTUP_IBMDS
extern void *_nx_st_ibmds_info();
#endif
extern void *_nx_st_mpl_info();
#endif /* NEXUS_LITE */
extern void *_nx_pr_mpl_info();
extern void *_nx_database_file_info();
static nexus_module_list_t default_module_list[] =
{
#ifndef NEXUS_LITE
#ifdef NEXUS_ARCH_STARTUP_IBMDS
    {"startups", "ibmds", _nx_st_ibmds_info},
#endif
    {"startups", "mpl", _nx_st_mpl_info},
#endif /* NEXUS_LITE */
    {"protocols", "mpl", _nx_pr_mpl_info},
    {"database", "file", _nx_database_file_info},
    {NULL, NULL, NULL},
};
#endif
#ifdef NEXUS_ARCH_MPINX
#define DEFINED_MODULE_LIST
#ifndef NEXUS_LITE
extern void *_nx_st_mpinx_info();
#endif /* NEXUS_LITE */
extern void *_nx_pr_mpinx_info();
extern void *_nx_database_file_info();
static nexus_module_list_t default_module_list[] =
{
#ifndef NEXUS_LITE
    {"startups", "mpinx", _nx_st_mpinx_info},
#endif /* NEXUS_LITE */
    {"protocols", "mpinx", _nx_pr_mpinx_info},
    {"database", "file", _nx_database_file_info},
    {NULL, NULL, NULL},
};
#endif
#ifdef NEXUS_ARCH_ATM
#define DEFINED_MODULE_LIST
extern void *_nx_st_fork_info();
extern void *_nx_st_rsh_info();
extern void *_nx_pr_atm_info();
extern void *_nx_database_file_info();
static nexus_module_list_t default_module_list[] =
{
#ifndef NEXUS_LITE
    {"startups", "fork", _nx_st_fork_info},
    {"startups", "rsh", _nx_st_rsh_info},
#endif /* NEXUS_LITE */
    {"protocols", "atm", _nx_pr_atm_info},
    {"database", "file", _nx_database_file_info},
    {NULL, NULL, NULL},
};
#endif
#ifdef NEXUS_ARCH_UDP
#define DEFINED_MODULE_LIST
extern void *_nx_st_fork_info();
extern void *_nx_st_rsh_info();
extern void *_nx_pr_udp_info();
extern void *_nx_database_file_info();
static nexus_module_list_t default_module_list[] =
{
#ifndef NEXUS_LITE
    {"startups", "fork", _nx_st_fork_info},
    {"startups", "rsh", _nx_st_rsh_info},
#endif /* NEXUS_LITE */
    {"protocols", "udp", _nx_pr_udp_info},
    {"database", "file", _nx_database_file_info},
    {NULL, NULL, NULL},
};
#endif
#ifdef NEXUS_ARCH_MN_UDP
#define DEFINED_MODULE_LIST
#ifndef NEXUS_LITE
extern void *_nx_st_fork_info();
extern void *_nx_st_rsh_info();
#endif /* NEXUS_LITE */
extern void *_nx_pr_mn_udp_info();
extern void *_nx_database_file_info();
static nexus_module_list_t default_module_list[] =
{
#ifndef NEXUS_LITE
    {"startups", "fork", _nx_st_fork_info},
    {"startups", "rsh", _nx_st_rsh_info},
#endif /* NEXUS_LITE */
    {"protocols", "mn_udp", _nx_pr_mn_udp_info},
    {"database", "file", _nx_database_file_info},
    {NULL, NULL, NULL},
};
#endif
#ifndef DEFINED_MODULE_LIST
#ifndef NEXUS_LITE
#ifdef NEXUS_ARCH_SOLARIS
extern void *_nx_st_soldl_info();
#endif
#ifdef NEXUS_ARCH_STARTUP_IBMDS
extern void *_nx_st_ibmds_info();
#endif
extern void *_nx_st_fork_info();
extern void *_nx_st_rsh_info();
extern void *_nx_st_ss_info();
#endif /* NEXUS_LITE */
extern void *_nx_pr_tcp_info();
extern void *_nx_database_file_info();
static nexus_module_list_t default_module_list[] =
{
#ifndef NEXUS_LITE
#ifdef NEXUS_ARCH_SOLARIS
/*    {"startups", "soldl", _nx_st_soldl_info}, */
#endif
#ifdef NEXUS_ARCH_STARTUP_IBMDS
    {"startups", "ibmds", _nx_st_ibmds_info},
#endif
    {"startups", "fork", _nx_st_fork_info},
    {"startups", "rsh", _nx_st_rsh_info},
    {"startups", "ss", _nx_st_ss_info},
#endif /* NEXUS_LITE */
    {"protocols", "tcp", _nx_pr_tcp_info},
    {"database", "file", _nx_database_file_info},
    {NULL, NULL, NULL},
};
#endif


/*
 * nexus_init()
 */
void nexus_init(int *argc,
		char ***argv,
		char *args_env_variable,
		char *package_designator,
		int (*package_args_init_func)(int *argc, char ***argv),
		void (*usage_message_func)(),
		int (*new_process_params_func)(char *buf, int size),
		nexus_module_list_t module_list[],
		nexus_node_t **nodes,
		int *n_nodes)
{
    _nx_NexusBoot = NexusBoot;
    _nx_init(argc,
	     argv,
	     args_env_variable,
	     package_designator,
	     package_args_init_func,
	     usage_message_func,
	     new_process_params_func,
	     (module_list ? module_list : default_module_list),
	     nodes,
	     n_nodes);
} /* nexus_init() */
