/*
 * mac_func.c
 *
 * Many of the Nexus functions can be efficiently implemented
 * as macros.  However, in some situations you may not want to use
 * the macros (i.e. when debugging).  If the user
 * defines NEXUS_USE_MACROS before including nexus.h, then the
 * macro versions will be used.
 *
 * This file holds the function versions of these various macros.
 */

static char *rcsid = "$Header: /Net/folding.compbio.caltech.edu/bcomp/src/master/nexus/mac_func.c,v 1.9 1995/06/06 23:48:21 tuecke Exp $";

#include "internal.h"


/*
 * nexus_null_global_pointer()
 *
 * Make 'gp' a NULL global pointer.
 */
#undef nexus_null_global_pointer
void nexus_null_global_pointer(nexus_global_pointer_t *gp)
{
    nexus_macro_null_global_pointer(gp);
} /* nexus_null_global_pointer() */


/*
 * nexus_is_null_global_pointer()
 *
 * Return NEXUS_TRUE if 'gp' is a NULL global pointer,
 * otherwise return NEXUS_FALSE.
 */
#undef nexus_is_null_global_pointer
nexus_bool_t nexus_is_null_global_pointer(nexus_global_pointer_t *gp)
{
    return (nexus_macro_is_null_global_pointer(gp));
} /* nexus_is_null_global_pointer() */


/*
 * nexus_init_remote_service_request()
 */
#undef nexus_init_remote_service_request
void nexus_init_remote_service_request(nexus_buffer_t *buffer,
				       nexus_global_pointer_t *gp,
				       char *handler_name,
				       int handler_id)
{
    nexus_macro_init_remote_service_request(buffer, gp,
					    handler_name, handler_id);
} /* nexus_init_remote_service_request() */


/*
 * nexus_set_buffer_size()
 */
#undef nexus_set_buffer_size
void nexus_set_buffer_size(nexus_buffer_t *buffer,
			   int size,
			   int n_elements)
{
    nexus_macro_set_buffer_size(buffer, size, n_elements);
} /* nexus_set_buffer_size() */


/*
 * nexus_check_buffer_size()
 */
#undef nexus_check_buffer_size
int nexus_check_buffer_size(nexus_buffer_t *buffer,
			    int slack,
			    int increment)
{
    return (nexus_macro_check_buffer_size(buffer, slack, increment));
} /* nexus_check_buffer_size() */


/*
 * nexus_send_remote_service_request()
 */
#undef nexus_send_remote_service_request
void nexus_send_remote_service_request(nexus_buffer_t *buffer)
{
    nexus_macro_send_remote_service_request(buffer);
} /* nexus_send_remote_service_request() */


/*
 * nexus_send_urgent_remote_service_request()
 */
#undef nexus_send_urgent_remote_service_request
void nexus_send_urgent_remote_service_request(nexus_buffer_t *buffer)
{
    nexus_macro_send_urgent_remote_service_request(buffer);
} /* nexus_send_urgent_remote_service_request() */


/*
 * nexus_free_buffer()
 */
#undef nexus_free_buffer
void nexus_free_buffer(nexus_buffer_t *buffer)
{
    nexus_macro_free_buffer(buffer);
} /* nexus_free_buffer() */


/*
 * nexus_stash_buffer()
 */
#undef nexus_stash_buffer
void nexus_stash_buffer(nexus_buffer_t *buffer,
			nexus_stashed_buffer_t *stashed_buffer)
{
    nexus_macro_stash_buffer(buffer, stashed_buffer);
} /* nexus_stash_buffer() */


/*
 * nexus_free_stashed_buffer()
 */
#undef nexus_free_stashed_buffer
void nexus_free_stashed_buffer(nexus_stashed_buffer_t *stashed_buffer)
{
    nexus_macro_free_stashed_buffer(stashed_buffer);
} /* nexus_free_stashed_buffer() */


/*
 * nexus_sizeof_TYPE()
 */
#undef nexus_sizeof_float
int nexus_sizeof_float(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_float(buffer, count));
}
#undef nexus_sizeof_double
int nexus_sizeof_double(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_double(buffer, count));
}
#undef nexus_sizeof_short
int nexus_sizeof_short(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_short(buffer, count));
}
#undef nexus_sizeof_u_short
int nexus_sizeof_u_short(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_u_short(buffer, count));
}
#undef nexus_sizeof_int
int nexus_sizeof_int(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_int(buffer, count));
}
#undef nexus_sizeof_u_int
int nexus_sizeof_u_int(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_u_int(buffer, count));
}
#undef nexus_sizeof_long
int nexus_sizeof_long(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_long(buffer, count));
}
#undef nexus_sizeof_u_long
int nexus_sizeof_u_long(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_u_long(buffer, count));
}
#undef nexus_sizeof_char
int nexus_sizeof_char(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_char(buffer, count));
}
#undef nexus_sizeof_u_char
int nexus_sizeof_u_char(nexus_buffer_t *buffer, int count)
{
    return (nexus_macro_sizeof_u_char(buffer, count));
}


/*
 * nexus_put_TYPE()
 */
#undef nexus_put_float
void nexus_put_float(nexus_buffer_t *buffer,
		     float *data,
		     int count)
{
    nexus_macro_put_float(buffer, data, count);
}
#undef nexus_put_double
void nexus_put_double(nexus_buffer_t *buffer,
		      double *data,
		      int count)
{
    nexus_macro_put_double(buffer, data, count);
}
#undef nexus_put_short
void nexus_put_short(nexus_buffer_t *buffer,
		     short *data,
		     int count)
{
    nexus_macro_put_short(buffer, data, count);
}
#undef nexus_put_u_short
void nexus_put_u_short(nexus_buffer_t *buffer,
		       unsigned short *data,
		       int count)
{
    nexus_macro_put_u_short(buffer, data, count);
}
#undef nexus_put_int
void nexus_put_int(nexus_buffer_t *buffer,
		   int *data,
		   int count)
{
    nexus_macro_put_int(buffer, data, count);
}
#undef nexus_put_u_int
void nexus_put_u_int(nexus_buffer_t *buffer,
		     unsigned int *data,
		     int count)
{
    nexus_macro_put_u_int(buffer, data, count);
}
#undef nexus_put_long
void nexus_put_long(nexus_buffer_t *buffer,
		    long *data,
		    int count)
{
    nexus_macro_put_long(buffer, data, count);
}
#undef nexus_put_u_long
void nexus_put_u_long(nexus_buffer_t *buffer,
		      unsigned long *data,
		      int count)
{
    nexus_macro_put_u_long(buffer, data, count);
}
#undef nexus_put_char
void nexus_put_char(nexus_buffer_t *buffer,
		    char *data,
		    int count)
{
    nexus_macro_put_char(buffer, data, count);
}
#undef nexus_put_u_char
void nexus_put_u_char(nexus_buffer_t *buffer,
		      unsigned char *data,
		      int count)
{
    nexus_macro_put_u_char(buffer, data, count);
}


/*
 * nexus_get_TYPE()
 */
#undef nexus_get_float
void nexus_get_float(nexus_buffer_t *buffer,
		     float *dest,
		     int count)
{
    nexus_macro_get_float(buffer, dest, count);
}
#undef nexus_get_double
void nexus_get_double(nexus_buffer_t *buffer,
		      double *dest,
		      int count)
{
    nexus_macro_get_double(buffer, dest, count);
}
#undef nexus_get_short
void nexus_get_short(nexus_buffer_t *buffer,
		     short *dest,
		     int count)
{
    nexus_macro_get_short(buffer, dest, count);
}
#undef nexus_get_u_short
void nexus_get_u_short(nexus_buffer_t *buffer,
		       unsigned short *dest,
		       int count)
{
    nexus_macro_get_u_short(buffer, dest, count);
}
#undef nexus_get_int
void nexus_get_int(nexus_buffer_t *buffer,
		   int *dest,
		   int count)
{
    nexus_macro_get_int(buffer, dest, count);
}
#undef nexus_get_u_int
void nexus_get_u_int(nexus_buffer_t *buffer,
		     unsigned int *dest,
		     int count)
{
    nexus_macro_get_u_int(buffer, dest, count);
}
#undef nexus_get_long
void nexus_get_long(nexus_buffer_t *buffer,
		    long *dest,
		    int count)
{
    nexus_macro_get_long(buffer, dest, count);
}
#undef nexus_get_u_long
void nexus_get_u_long(nexus_buffer_t *buffer,
		      unsigned long *dest,
		      int count)
{
    nexus_macro_get_u_long(buffer, dest, count);
}
#undef nexus_get_char
void nexus_get_char(nexus_buffer_t *buffer,
		    char *dest,
		    int count)
{
    nexus_macro_get_char(buffer, dest, count);
}
#undef nexus_get_u_char
void nexus_get_u_char(nexus_buffer_t *buffer,
		      unsigned char *dest,
		      int count)
{
    nexus_macro_get_u_char(buffer, dest, count);
}


/*
 * nexus_get_stashed_TYPE()
 */
#undef nexus_get_stashed_float
void nexus_get_stashed_float(nexus_stashed_buffer_t *buffer,
			     float *dest,
			     int count)
{
    nexus_macro_get_stashed_float(buffer, dest, count);
}
#undef nexus_get_stashed_double
void nexus_get_stashed_double(nexus_stashed_buffer_t *buffer,
			      double *dest,
			      int count)
{
    nexus_macro_get_stashed_double(buffer, dest, count);
}
#undef nexus_get_stashed_short
void nexus_get_stashed_short(nexus_stashed_buffer_t *buffer,
			     short *dest,
			     int count)
{
    nexus_macro_get_stashed_short(buffer, dest, count);
}
#undef nexus_get_stashed_u_short
void nexus_get_stashed_u_short(nexus_stashed_buffer_t *buffer,
			       unsigned short *dest,
			       int count)
{
    nexus_macro_get_stashed_u_short(buffer, dest, count);
}
#undef nexus_get_stashed_int
void nexus_get_stashed_int(nexus_stashed_buffer_t *buffer,
			   int *dest,
			   int count)
{
    nexus_macro_get_stashed_int(buffer, dest, count);
}
#undef nexus_get_stashed_u_int
void nexus_get_stashed_u_int(nexus_stashed_buffer_t *buffer,
			     unsigned int *dest,
			     int count)
{
    nexus_macro_get_stashed_u_int(buffer, dest, count);
}
#undef nexus_get_stashed_long
void nexus_get_stashed_long(nexus_stashed_buffer_t *buffer,
			    long *dest,
			    int count)
{
    nexus_macro_get_stashed_long(buffer, dest, count);
}
#undef nexus_get_stashed_u_long
void nexus_get_stashed_u_long(nexus_stashed_buffer_t *buffer,
			      unsigned long *dest,
			      int count)
{
    nexus_macro_get_stashed_u_long(buffer, dest, count);
}
#undef nexus_get_stashed_char
void nexus_get_stashed_char(nexus_stashed_buffer_t *buffer,
			    char *dest,
			    int count)
{
    nexus_macro_get_stashed_char(buffer, dest, count);
}
#undef nexus_get_stashed_u_char
void nexus_get_stashed_u_char(nexus_stashed_buffer_t *buffer,
			      unsigned char *dest,
			      int count)
{
    nexus_macro_get_stashed_u_char(buffer, dest, count);
}


#ifdef NEXUS_LITE
/*
 * nexus_mutex_init()
 */
#undef nexus_mutex_init
int nexus_mutex_init(nexus_mutex_t *mut, nexus_mutexattr_t *attr)
{
    return (nexus_macro_mutex_init(mut, attr));
} /* nexus_mutex_init() */


/*
 *  nexus_mutex_destroy()
 */
#undef nexus_mutex_destroy
int nexus_mutex_destroy(nexus_mutex_t *mut)
{
    return (nexus_macro_mutex_destroy(mut));
} /* nexus_mutex_destroy() */


/* 
 *  nexus_mutex_lock()
 */
#undef nexus_mutex_lock
int nexus_mutex_lock(nexus_mutex_t *mut)
{
    return (nexus_macro_mutex_lock(mut));
} /* nexus_mutex_lock() */


/*
 *  nexus_mutex_unlock()
 */
#undef nexus_mutex_unlock
int nexus_mutex_unlock(nexus_mutex_t *mut)
{
    return (nexus_macro_mutex_unlock(mut));
} /* nexus_mutex_unlock() */


/*
 * nexus_cond_init()
 */
#undef nexus_cond_init
int nexus_cond_init(nexus_cond_t *cv, nexus_condattr_t *attr)
{
    return (nexus_macro_cond_init(cv, attr));
} /* nexus_cond_init() */


/*
 *  nexus_cond_destroy()
 */
#undef nexus_cond_destroy
int nexus_cond_destroy(nexus_cond_t *cv)
{
    return (nexus_macro_cond_destroy(cv));
} /* nexus_cond_destroy() */


/*
 *  nexus_cond_wait()
 */
#undef nexus_cond_wait
int nexus_cond_wait(nexus_cond_t *cv, nexus_mutex_t *mut)
{
    return (nexus_macro_cond_wait(cv, mut));
} /* nexus_cond_wait() */


/*
 *  nexus_cond_signal()
 */
#undef nexus_cond_signal
int nexus_cond_signal(nexus_cond_t *cv)
{
    return (nexus_macro_cond_signal(cv));
} /* nexus_cond_signal () */


/*
 *  nexus_cond_broadcast()
 */
#undef nexus_cond_broadcast
int nexus_cond_broadcast(nexus_cond_t *cv)
{
    return (nexus_macro_cond_broadcast(cv));
} /* nexus_cond_broadcast() */

#endif /* NEXUS_LITE */
