/*
 * st_ss.c
 *
 * Remote node startup using secure server
 */

static char *rcsid = "$Header: /Net/folding.compbio.caltech.edu/bcomp/src/master/nexus/st_ss.c,v 1.10 1995/07/07 19:55:57 patton Exp $";

#include "internal.h"
#include "server.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pwd.h>

#if defined(NEXUS_ARCH_AIX) 
#include <sys/select.h>
#include <sys/ioctl.h>
#include <unistd.h>
#endif

#ifdef NEXUS_ARCH_AIX4
#include <sys/signal.h>
#endif

#ifdef NEXUS_ARCH_SUNOS41
#include <sys/time.h>
#endif

#ifdef NEXUS_DEBUG
#define nexus_dbgss_printf( operation, level, message ) nexus_new_debug_printf( NXDBG_ST, NXDBG_ST_SS, operation, level, message )
#else
#define nexus_dbgss_printf( operation, level, message )
#endif /* NEXUS_DEBUG */
#define NXDBG_SS_NODE 0x00000001

static void	st_ss_start_node(nexus_startup_node_t *node,
				  unsigned long context,
				  unsigned long address,
				  unsigned long class_type,
				  int first_checkin_number);
static nexus_bool_t    start_secure_remote_node(char *host, int port, 
			    	  char *username,
				  char *dir, char *cmd, char *args);

static int connect_to_server(char *host, int port);
static void send_string(int sock, char *str);
static void recv_string(int sock, char *str, int len);
static void setup_anon_listener(int backlog, int *port, int *sock);
static char *getpw_ss(char *host, char *username);
static int echo(void);

static nexus_startup_funcs_t st_ss_funcs =
{
    NULL /* st_ss_preinit */,
    NULL /* st_ss_get_master_node_name */,
    NULL /* st_ss_init */,
    NULL /* st_ss_shutdown */,
    NULL /* st_ss_abort */,
    st_ss_start_node,
    NULL /* st_ss_start_context */,
};

extern char *  _nx_debug_command;
extern char *  _nx_debug_display;

/*
 * _nx_st_ss_info()
 *
 * Return the function table for this module.
 */
void *_nx_st_ss_info()
{
    return ((void *) (&st_ss_funcs));
}


/*
 * st_ss_start_node()
 *
 * Start the node(s) for the passed nexus_startup_node_t structure.
 */
static void st_ss_start_node(nexus_startup_node_t *node,
			      unsigned long context,
			      unsigned long address,
			      unsigned long class_type,
			      int first_checkin_number)
{
    char args[NEXUS_MAX_COMMAND_LENGTH], args2[NEXUS_MAX_COMMAND_LENGTH];
    int args2_len;
    int i;
    char *port;
    char *username;
    char *listener_path;

    nexus_dbgss_printf( NXDBG_SS_NODE, 1, ("st_ss_start_node(): starting node %s#%d\n", node->name, node->number));

    for (i = 0; i < node->count; i++)
    {
	/* Get all arguments that need to go to this process */
	_nx_new_process_params(args2, NEXUS_MAX_COMMAND_LENGTH);
	args2_len = strlen(args2);

	_nx_proto_get_creator_gp_params(args2 + args2_len,
					NEXUS_MAX_COMMAND_LENGTH - args2_len,
					node->name,
					node->number,
					NULL,
					context,
					address,
					class_type);

	nexus_stdio_lock();
	sprintf(args,
		"%s -nx_node %s %d n -nx_node_id %d -nx_checkin %d %s %s",
		_nx_get_package_id_start(),
		node->name,
		node->number + i,
		node->id + i,
		first_checkin_number + i,
		args2,
		_nx_get_package_id_end() );
	nexus_stdio_unlock();

	port = nexus_database_lookup(node->name,
				     node->number,
				     "ss_port");

	if( port ) nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("Using port %s\n", port));
	username = nexus_database_lookup(node->name,
					 node->number,
					 "ss_login");

	if( username ) nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("Using username %s\n", username ));

	if ((listener_path = getenv("NEXUS_LISTENER")) != (char *)NULL)
	{ 	    
	    nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("Using listener %s from environment variable\n", listener_path)); 
	}
	else if ((listener_path = nexus_database_lookup(node->name,
							node->number,
							"ss_listener")) != (char *)NULL) 
	{
	    nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("Using listener %s from database\n", listener_path)); 
	}
	else if((listener_path = _nx_path_array[NEXUS_PATH_TYPE_LISTENER]) != (char *)NULL) {
	    nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("Using listener %s from path array\n",listener_path));
	} else {
	    listener_path = NEXUS_LISTENER;
	}

	if (!username)
	{
	    username = _nx_copy_string("NEXUS");
	}
/* JGG
 *
 * Currently, it is not safe just to call start_secure_remote_node()
 * within some multi-threaded environments because _nx_md_fork()
 * replicates all the threads in the new process instead of just the
 * current one.  So what happens is that the listener does a context
 * switch to the handler thread and the handler thread looks for
 * messages that it is never going to get and blocks all the other
 * threads ending in a deadlock.
 *
 * If this gets fixed, a (close) mirror of listener.c exists below and
 * should be able to be used without incident.  To keep things
 * consistent, though, we will use one listener piece of code.
 */
#if defined(JGG)
	start_secure_remote_node(node->name, port, username, node->directory_path, node->executable_path, args);
#else
	nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("st_ss_start_node(): fork/exec <%s %s %s %s %s %s '%s'>\n",listener_path, node->name, port, username, node->directory_path, node->executable_path, args));
	
	if (_nx_dont_start_processes)
	{
	    nexus_printf("Would run: <%s %s %s %s %s %s '%s'>\n", listener_path, node->name, port, username, node->directory_path, node->executable_path, args);
	}
	else
	{
	    if (!_nx_md_fork())
	    {
		char tmp1[512];
		sprintf(tmp1,"%s/%s", listener_path, "listener");
		execlp(/*listener_path*/ tmp1, "listener", node->name, port, username, node->directory_path, node->executable_path, args, NULL);
		nexus_fatal("st_ss_start_node(): exec failed:%d\n",errno);
	    }
	}
#endif
    }
	
} /* st_rsh_start_node() */


/*
 * start_secure_remote_node()
 */
static nexus_bool_t start_secure_remote_node(char *host, int port, char *remote_username, char *dir, char *cmd, char *args)
{
    char myhost[256];
    int conn;
    struct passwd *passwd;
    char *local_username;
    struct timeval timeout;
    fd_set recv_fds;
    char buf[250];
    char *pw_string;
    int pid;
    int new_port, new_fd;
    char port_string[50];
    int stdout_fd;
    struct sockaddr_in temp;
    int templen;
    int n;
    char msg[500];

    if (_nx_dont_start_processes)
    {
	nexus_printf("Would run <%s %s> on host %s\n",
		     cmd, args, host);
	return NEXUS_TRUE;
    }

    /* practically copied from p4_secure.c in the MPIch release */

    myhost[0] = '\0';
    gethostname(myhost, sizeof(myhost));
    
    nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("start_secure_node(): opening connection\n"));

    conn = connect_to_server(host, port);
    if (conn < 0)
    {
	nexus_dbgss_printf( NXDBG_SS_NODE, 2, ("start_secure_node(): connect failed\n"));
	return NEXUS_FALSE;
    }

    nexus_dbgss_printf( NXDBG_SS_NODE, 2, ("start_secure_node(): Connected\n"));

    passwd = getpwuid(geteuid());
    if (passwd == NULL)
    {
	local_username = getlogin();
	if (local_username == NULL)
	{
	    nexus_warning("No local username found\n");
	    return NEXUS_FALSE;
	}
    }
    else
    {
	local_username = passwd->pw_name;
    }

    send_string(conn, local_username);
    send_string(conn, remote_username);

    timeout.tv_sec = 1;
    timeout.tv_usec = 0;
    FD_ZERO(&recv_fds);
    FD_SET(conn, &recv_fds);
/* JGG
 *
 *  The original code looked like:
 *
 *      n = select(conn+1, &recv_fds, NULL, NULL, &timeout);
 *      if (n == 1)
 *      {
 *          recv_string(conn, buf, sizeof(buf));
 *      }
 *      else
 *      {
 *          nexus_fatal("Wrong number of new connections\n");
 *      }
 *
 *  The comment was added because of the multi-threaded problem of a
 *  select being interrupted by another thread.  It should be enough
 *  to let the recv_string() figure out when the first message comes,
 *  but I'm keeping this here "just in case."
 */
    recv_string(conn, buf, sizeof(buf));

    nexus_dbgss_printf( NXDBG_SS_NODE, 2, ("start_secure_remote_node(): Got reply1 %s\n", buf));

    if (strncmp(buf, "Password", 8) == 0)
    {
        pw_string = getpw_ss(host, remote_username);

	send_string(conn, pw_string);
	recv_string(conn, buf, sizeof(buf));
	NexusFree(pw_string);

	nexus_dbgss_printf( NXDBG_SS_NODE, 2, ("start_secure_remote_node(): Got reply2 %s\n", buf));

	if (strncmp(buf, "Proceed", 7) != 0)
	{
	    return NEXUS_FALSE;
	}
    }
    else
    {
	if (strncmp(buf, "Proceed", 7) != 0)
	{
	    return NEXUS_FALSE;
	}
    }

    nexus_dbgss_printf( NXDBG_SS_NODE, 2, ("start_secure_remote_node(): Validated on remote system\n"));

    send_string(conn, SERVER_CD_NOTIFIER);
    send_string(conn, dir);
    send_string(conn, cmd);
    send_string(conn, args);

    if ((pid = _nx_md_fork()) == 0)
    {
	setup_anon_listener(10, &new_port, &new_fd);
	sprintf(port_string, "%d", new_port);
	send_string(conn, port_string);
	fflush(_nx_stdout);
	templen = sizeof(temp);

	stdout_fd = accept(new_fd, (struct sockaddr *)&temp, &templen);
	close(new_fd);

	n = 1;
	while (n > 0)
	{
	    n = read(stdout_fd, msg, sizeof(msg)-1);
	    if (n > 0)
	    {
		msg[n] = '\0';
		nexus_stdio_lock();
		printf("%s\n", msg);
		nexus_stdio_unlock();
	    }
	}
        exit(0);
    }

    recv_string(conn, buf, sizeof(buf));

    nexus_dbgss_printf( NXDBG_SS_NODE, 2, ("start_secure_remote_node(): Got reply3 %s\n", buf));

    if (strncmp(buf, "Success", 7) != 0)
    {
	kill(pid, SIGKILL);
	return NEXUS_FALSE;
    }

    close(conn);

    return NEXUS_TRUE;
} /* start_secure_remote_node() */

static int connect_to_server(char *host, int server_port)
{
    int conn;
    struct hostent *hostent;
    struct sockaddr_in addr;
    int rc;

    conn = socket(AF_INET, SOCK_STREAM, 0);
    if (conn < 0)
    {
	return -1;
    }

    hostent = gethostbyname(host);

    addr.sin_family = hostent->h_addrtype;
    addr.sin_port = htons(server_port);
    memcpy(&addr.sin_addr, hostent->h_addr, hostent->h_length);

    rc = connect(conn, (struct sockaddr *)&addr, sizeof(addr));
    if (rc < 0)
    {
	return -1;
    }

    return conn;
}

static void send_string(int sock, char *str)
{
    int rc;
    int len = strlen(str);
    char nl = 10;

    rc = write(sock, str, len);
    if (rc < 0)
    {
	nexus_fatal("send_string():  write to secure server failed\n");
    }

    rc = write(sock, &nl, 1);
    if (rc < 0)
    {
	nexus_fatal("send_string():  write to secure server failed\n");
    }
    
    nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("send_string(): sent \"%s\" successfully\n", str));
}

static void recv_string(int sock, char *buf, int len)
{
    char *bptr;
    int n;

    bptr = buf;
    while(NEXUS_TRUE)
    {
	n = read(sock, bptr, 1);
	if (n < 0)
	{
	    nexus_fatal("recv_string(): read from secure server failed\n");
	}

	if (*bptr == '\n') break;

	bptr++;
	if (bptr - buf >= len) break;
    }
    *bptr = 0;
    nexus_dbgss_printf( NXDBG_SS_NODE, 3, ("recv_string(): read \"%s\" successfully\n", buf));
}

static void setup_anon_listener(int backlog, int *port, int *sock)
{
    int sin_len;
    struct sockaddr_in sin;
    int rc;

    *sock = socket(AF_INET, SOCK_STREAM, 0);
    if (*sock < 0)
    {
	nexus_fatal("setup_anon_listener(): unable to setup a listener socket\n");
    }

    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;
    sin.sin_port = htons(0);

    sin_len = sizeof(sin);

    rc = bind (*sock, (struct sockaddr *)&sin, sin_len);
    if (rc < 0)
    {
	nexus_fatal("setup_anon_listener(): unable to bind socket\n");
    }

    rc = listen (*sock, backlog);
    if (rc < 0)
    {
	nexus_fatal("setup_anon_listener(): unable to listen on to socket\n");
    }

    getsockname(*sock, (struct sockaddr *)&sin, &sin_len);
    *port = ntohs(sin.sin_port);

}

static char *getpw_ss(char *host, char *username)
{
    char *buf;
    char *s;

    NexusMalloc(getpw_ss(),
		buf,
		char *,
		sizeof(char) * 100);
    echo();
    printf("Password for %s@%s:  ", username, host);
    fflush(_nx_stdout);
    fgets(buf, 100, stdin);
    echo();
    printf("\n");

    for (s = buf; *s; s++)
    {
	if (*s == '\n')
	{
	    *s = '\0';
	    break;
	}
    }

    return buf;
}

#if defined(NEXUS_ARCH_NEXT040)
static int echo()
{
    struct sgttyb orig_tty, tty_new;

    if (ioctl(0, TIOCGETP, &orig_tty) < 0)
    {
	nexus_warning("ioctl() TIOCGETP failed\n");
	return -1;
    }
    
    tty_new = orig_tty;
    tty_new.sg_flags &= ~(ECHO);

    if (ioctl(0, TIOCSETP, &tty_new) < 0)
    {
	nexus_warning("ioctl() TIOSETP failed\n");
	return -1;
    }
    return 0;
}
#else

#include <termio.h>

static int echo()
{
    struct termio tty_orig, tty_new;

    if (ioctl(0, TCGETA, &tty_orig) < 0)
    {
	nexus_warning("ioctl() TCGETA failed\n");
	return -1;
    }

    tty_new = tty_orig;
    tty_new.c_lflag &= ~(ECHO);

    if (ioctl(0, TCSETA, &tty_new) < 0)
    {
	nexus_warning("ioctl() TCSETA failed\n");
	return -1;
    }

    return 0;
}
#endif
